'### CSSoundSample ###

Type CSAudioSample
	
	Field length:Int
	Field format:Int
	Field rate:Float
	Field samples:Byte Ptr
	
	Rem
	bbdoc: Returns a new empty CSAudioSample object.
	EndRem
	Function audioSample:CSAudioSample()
		Return New CSAudioSample
	EndFunction
	
	Rem
	bbdoc: Returns a new empty CSAudioSample object.
	EndRem
	Function Create:CSAudioSample()
		Return CSAudioSample.audioSample()
	EndFunction
	
	Rem
	bbdoc: Returns a new instance of CSAudioSample with allocated samples.
	EndRem
	Function withSize:CSAudioSample(size_:Int)
		Local ret:CSAudioSample = New CSAudioSample
			ret.alloc(size_)
		Return ret
	EndFunction
	
	Rem
	bbdoc: Reuturns a new instance of CSAudioSample with the data of a TAudioSample
	EndRem
	Function fromTAudioSample:CSAudioSample( sample_:TAudioSample )
		Local ret:CSAudioSample = New CSAudioSample
			ret.alloc(sample_.length*AudioFormatMonoOrStereo(sample_.format))
			MemCopy(ret.samples, sample_.samples, sample_.length*AudioFormatMonoOrStereo(sample_.format))
			ret.format = AudioFormatToALFormat(sample_.format)
			ret.rate = sample_.hertz
			ret.length = sample_.length
		Return ret
	EndFunction
	
	Method alloc( size_:Int )
		MemFree Self.samples
		samples = MemAlloc(size_)
	EndMethod
	
	Method Delete()
		MemFree Self.samples
	EndMethod
	
	'------------------
	'splitting addition
	'------------------
	
	Method trimSample( length_:Long )
		'later
	EndMethod
	
EndType

'------
'loader
'------

Private
Function readtag:Int( stream_:TStream, tag_:String )
	Local str:String = ReadString(stream_, 4)
	If str = tag_
		Return True
	Else
		Return False
	EndIf
EndFunction

Function CSBytespersample:Int( format_:Int )
	Select(format_)
		Case AL_FORMAT_MONO8
			Return 1
		Case AL_FORMAT_MONO16
			Return 2
		Case AL_FORMAT_STEREO8
			Return 2
		Case AL_FORMAT_STEREO16
			Return 4
		Default
			Return 1
	EndSelect
EndFunction

Function AudioFormatToALFormat:Int( format_:Int )
	Select format_
		Case SF_MONO8
			Return AL_FORMAT_MONO8
		Case SF_MONO16LE
			Return AL_FORMAT_MONO16
		Case SF_MONO16BE
			Return AL_FORMAT_MONO16
		Case SF_STEREO8
			Return AL_FORMAT_STEREO8
		Case SF_STEREO16LE
			Return AL_FORMAT_STEREO16
		Case SF_STEREO16BE
			Return AL_FORMAT_STEREO16
	EndSelect
EndFunction

Function AudioFormatMonoOrStereo:Int( format_:Int )
	Select format_
		Case AL_FORMAT_MONO8
			Return 1
		Case SF_MONO16LE
			Return 2
		Case SF_MONO16BE
			Return 2
		Case SF_STEREO8
			Return 2
		Case SF_STEREO16LE
			Return 4
		Case SF_STEREO16BE
			Return 4
		Default 
			Return 1
	EndSelect
EndFunction
Public

Rem
bbdoc: Loads an OGG or WAV file into a CSAudoSample,<br> loading is done by TAudioSample.
EndRem
Function CSLoad:CSAudioSample( path_:String )
	Return CSAudioSample.fromTAudioSample(LoadAudioSample(path_))
EndFunction

Rem
bbdoc: Loads an WAV file into a CSAudioSample.
EndRem
Function CSLoadWAV:CSAudioSample( path_:String )
	Local stream:TStream = LittleEndianStream(ReadStream(path_))
	Assert stream, "File not found"
	
	Local length:Int
	Local length2:Int
	Local comp:Short
	
	If Not readtag(stream, "RIFF") Return Null
	length = ReadInt(stream)-8
	
	If Not readtag(stream, "WAVE") Return Null
	If Not readtag(stream, "fmt ") Return Null
	length2 = ReadInt(stream)
	comp = ReadShort(stream)
	If comp<>1 Return Null
	
	Local channels:Int = ReadShort(stream)
	Local rate:Int = ReadInt(stream)
	Local bytessec:Int = ReadInt(stream)
	Local pad:Short = ReadShort(stream)
	Local bits:Short = ReadShort(stream)
	Local format:Int = 0
	
	If bits=8 And channels=1
		format = AL_FORMAT_MONO8
	ElseIf bits=8 And channels=2
		format = AL_FORMAT_STEREO8
	ElseIf bits=16 And channels=1
		format = AL_FORMAT_MONO16
	ElseIf bits=16 And channels=2
		format = AL_FORMAT_STEREO16
	Else
		Return Null
	EndIf
	
	If length2 > 16 stream.skipbytes(length2-16)
	
	If Not Eof(stream)
		While Not readtag(stream, "data")
			Local skip:Int = stream.ReadInt()
			stream.skipbytes(skip)
		Wend
		
		Local bytes:Int = stream.ReadInt()
		Local samplelength:Int = bytes/CSBytespersample(format)
		Local sample:CSAudioSample = CSAudioSample.audioSample()
			sample.length = samplelength
			sample.rate = rate
			sample.format = format
			sample.alloc(bytes)
		stream.readbytes(sample.samples, bytes)
		
		Return sample
	Else
		Return Null
	EndIf
	
EndFunction