'### coresound.sound ###'

Rem
bbdoc: CSSound provides simple sound playback and time functions.
EndRem
Type CSSound

	Global sounds:TList

	Field _buffer:Int[1]
	Field _source:Int[1]
	
	Field _bufferCount:Int
	
	Field _byte:Int
	Field _length:Int
	Field _format:Int
	Field _hertz:Double
	
	'addition
	Field _audioSample:CSAudioSample
		
	'------
	'create
	'------
	
	Rem
	bbdoc: Deprecated! Creates a new empty (but initialized) CSSound instance.<br>init must be called to use!
	EndRem
	Function Create:CSSound()
		Local ret:CSSound = New CSSound
			ret.init()
		Return ret
	EndFunction
	
	Rem
	bbdoc: Creates a new empty (but initialized) CSSound instance.<br>init must be called to use!
	EndRem
	Function sound:CSSound()
		Local ret:CSSound = New CSSound
			ret.init()
		Return ret
	EndFunction
	
	Rem
	bbdoc: Initializes the CSSound instance.
	EndRem
	Method init()
		If Not CSSound.sounds CSSound.sounds = CreateList() 
		generateBuffers(1)
		generateSources()
		CSSound.sounds.addLast(Self)
	EndMethod
	
	'---------------
	'buffer & source
	'---------------
	
	Method generateBuffers( count_:Int=1 )
		alGetError()
		_buffer = New Int[count_]
		_buffercount = count_
		alGenBuffers( count_, _buffer )
		If alGetError() CoreSoundError("Error generating buffer!")
	EndMethod
	
	Method generateSources()
		alGetError()
		alGenSources( 1, _source )
		If alGetError() CoreSoundError("Error generating source!")
	EndMethod
	
	'---------
	'set & get
	'---------
	
	Rem
	bbdoc: Returns the sounds source.
	EndRem
	Method getSource:Int()
		Return _source[0]
	EndMethod
	
	Rem
	bbdoc: Returns the sounds audio buffer.
	EndRem
	Method getBuffer:Int( i_:Int )
		Return _buffer[i_]
	EndMethod
	
	Rem
	bbdoc: Returns the TAudioSample, if it was loaded.
	EndRem
	Method getAudioSample:CSAudioSample()
		Return _audiosample
	EndMethod
	
	Rem
	bbdoc: Returns the sound rate (hertz).
	EndRem
	Method getRate:Double()
		Return _hertz
	EndMethod
	
	Rem
	bbdoc: Returns the sounds pitch value.
	EndRem
	Method getPitch:Float()
		Local ret_:Float
		alGetSourcef( _source[0], AL_PITCH, Varptr(ret_) )
		Return ret_
	EndMethod
	
	Rem
	bbdoc: Sets the sound pitch (1.0).
	EndRem
	Method setPitch( pitch_:Float = 1.0 )
		alSourcef( _source[0], AL_PITCH, pitch_ )
	EndMethod
	
	Rem
	bbdoc: Returns the sounds gain.
	EndRem
	Method getGain:Float()
		Local ret_:Float
		alGetSourcef( _source[0], AL_GAIN, Varptr(ret_) )
		Return ret_
	EndMethod
	
	Rem
	bbdoc: Set the sounds gain (1.0).
	EndRem
	Method setGain( gain_:Float = 1.0 )
		alSourcef( _source[0], AL_GAIN, gain_ )
	EndMethod
	
	Rem
	bbdoc: Set if the sound should loop.
	EndRem
	Method getLoop:Int()
		Local ret_:Int
		alGetSourcei( _source[0], AL_LOOPING, Varptr(ret_) )
		Return ret_
	EndMethod
	
	Rem
	bbdoc: Returns if looping is on.
	EndRem
	Method setLoop( loop_:Int )
		alSourcei( _source[0], AL_LOOPING, loop_ )
	EndMethod
	
	'----
	'load
	'----
	
	Rem
	bbdoc: load an existing audio sample into the sounds buffer
	EndRem
	Method loadSample( snd_:CSAudioSample )
		If Not snd_ CoreSoundError( "Error loading from sample" )
		
		_hertz = snd_.rate
		_format = snd_.format
		_length = snd_.length
		_byte = CSBytespersample(snd_.format)
		
		alGetError()
		alBufferData( _buffer[0], _format, snd_.samples, _length * _byte, snd_.rate )
		If alGetError() CoreSoundError( "Error buffering data!" )
		
		alGetError()
		alSourceQueueBuffers( _source[0], _bufferCount, _buffer )
		If alGetError() CoreSoundError( "Error queing buffers!" )
		
		_audiosample = snd_
	EndMethod
	
	Rem
	bbdoc: Load a sound (wav) into a CSSound.
	EndRem
	Method LoadWAV( url_:String, saveData_:Int=False)
		Local snd_:CSAudioSample = CSLoadWAV(url_)
		If Not snd_ CoreSoundError( "Error loading sound!" )
		
		_hertz = snd_.rate
		_format = snd_.format
		_length = snd_.length
		_byte = CSBytespersample(snd_.format)
		
		alGetError()
		alBufferData( _buffer[0], _format, snd_.samples, _length * _byte, snd_.rate )
		If alGetError() CoreSoundError( "Error buffering data!" )
		
		alGetError()
		alSourceQueueBuffers( _source[0], _bufferCount, _buffer )
		If alGetError() CoreSoundError( "Error queing buffers!" )
		
		If saveData_ = True
			_audiosample = snd_
		Else
			'destroy the taudiosample
			snd_ = Null
			GCCollect()
		EndIf
	EndMethod
	
	'------------
	'play / pause
	'------------
	
	Rem
	bbdoc: Plays the source.
	EndRem
	Method play()
		alGetError()
		If _source[0] Then alSourcePlay( _source[0] )
	EndMethod
	
	Rem
	bbdoc: Pauses the source.
	EndRem
	Method pause()
		alGetError()
		If _source[0] Then alSourcePause( _source[0] )
	EndMethod
	
	Rem
	bbdoc: Stops the source.
	EndRem
	Method stop()
		alGetError()
		If _source[0] Then alSourceStop( _source[0] )
	EndMethod
	
	Rem
	bbdoc: Rewinds the source.
	EndRem
	Method rewind()
		alGetError()
		If _source[0] Then alSourceRewind( _source[0] )
	EndMethod
	
	Rem
	bbdoc: Returns if the source is playing
	EndRem
	Method isPlaying:Int()
		Local state_:Int
		alGetSourcei(_source[0], AL_SOURCE_STATE, Varptr(state_))     
		Return state_ = AL_PLAYING
	EndMethod
	
	'--------
	'position
	'--------
	
	Rem
	bbdoc: Return the sample offset.
	EndRem
	Method getSampleOffset:Long()
		Local ret_:Int
		alGetSourcei( _source[0], AL_SAMPLE_OFFSET, Varptr(ret_) )
		Return ret_
	EndMethod
	
	Rem
	bbdoc: Set the current time in samples.
	EndRem
	Method setSampleOffset( smp_:Int )
		alSourcei( _source[0], AL_SAMPLE_OFFSET, smp_ )
	EndMethod
	
	Rem
	bbdoc: Return the duration in sec (float).
	EndRem
	Method getTime:Float()
		Return (_length/_hertz)
	EndMethod
	
	Rem
	bbdoc: Return the time as in seconds (float).
	EndRem
	Method getCurrentTime:Float()
		Return (getSampleOffset()/_hertz)
	EndMethod
	
	Rem
	bbdoc: Set the time in second s(float).
	EndRem
	Method setCurrentTime( time_:Float )
		setSampleOffset(time_*_hertz)
	EndMethod

	Rem
	bbdoc: Return the current time as string. (min:sec:msec)
	EndRem
	Method getCurrentTimeString:String()
		Return Int(getCurrentTime()/60.0) + ":" + Int(getCurrentTime() Mod 60) + ":" + String(getCurrentTime() - Int(getCurrentTime()))[2..5]
	EndMethod
	
	Rem
	bbdoc: Return duration as string. (min:sec:msec)
	EndRem
	Method getTimeString:String()
		Return Int(getTime()/60.0) + ":" + Int(getTime() Mod 60) + ":" + String(getTime() - Int(getTime()))[2..5]
	EndMethod

	'-------
	'destroy
	'-------
	
	Rem
	bbdoc: Auto called for every CSSound when the application will quit.
	EndRem
	Method deinit()
		alSourceUnqueueBuffers(_source[0], _bufferCount, _buffer)
		alDeleteSources( 1, _source )
		alDeleteBuffers( _bufferCount, _buffer )
	EndMethod

EndType
